# -*- coding: utf-8 -*-
import swapper
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from jmb.filters.admin import AdvancedSearchModelAdmin
from thx.core.admin import StatusActionMixin, UserAdminMixin

from . import models

Contact = swapper.load_model('organization', 'Contact')
Organization = swapper.load_model('organization', 'Organization')


class ContactInline(admin.StackedInline):
    model = swapper.load_model('organization', 'Contact')
    extra = 0
    exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'status', 'code', 'user', 'title',
                ('last_name', 'first_name'),
                'roles', 'email', 'note', 'ordering',
            ),
        }),
    )
    autocomplete_fields = ('user', 'roles',)


class OrganizationAddressInline(admin.StackedInline):
    model = models.OrganizationAddress
    extra = 0
    exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'status', 'address_scopes', ('address_type', 'name'),
                ('number', 'zip_code'), ('locality', 'province'),
                'country', 'note'
            )
        }),
    )
    autocomplete_fields = ('address_scopes', 'country',)


class OrganizationPhoneInline(admin.TabularInline):
    model = models.OrganizationPhone
    extra = 0
    exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'phone', 'is_primary',)
        }),
    )


class OrganizationEmailInline(admin.TabularInline):
    model = models.OrganizationEmail
    extra = 0
    exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'email', 'is_primary',)
        }),
    )


class OrganizationSocialPageInline(admin.TabularInline):
    model = models.OrganizationSocialPage
    extra = 0
    exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier',)
    fieldsets = (
        (_('general information'), {
            'fields': ('social', 'link',)
        }),
    )


class OrganizationAddressAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ()
    advanced_search_fields = ()
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'organization', 'org_name', 'address_scopes', 'address_type', 'name', 'number',
                'zip_code', 'locality', 'province', 'country',
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = (
        'id', 'status', 'organization', 'org_name', 'address_type', 'name', 'number',
        'zip_code', 'locality', 'province', 'country', 'ordering',
    )
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('organization', 'address_scopes', 'country',)


class EmailTypologyAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name', 'slug',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'code', 'name', 'slug', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    prepopulated_fields = {'slug': ('name',)}


class PhoneTypologyAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name', 'slug',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'code', 'name', 'slug', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    prepopulated_fields = {'slug': ('name',)}


class OrganizationTypeAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('category', 'code', 'name', 'description',),
        }),
        (_('note'), {
            'fields': ('note',),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'code', 'name', 'category', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class OrganizationFormAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name', 'description',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'code', 'name', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class OrganizationAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('name',)
    advanced_search_fields = (
        ('status', 'organization_type__category',),
        ('code', 'name__icontains', 'name__startswith',),
        ('contacts__last_name__icontains', 'contacts__first_name__icontains'),
        ('organization_type', 'organization_form',),
        # ('vat',),
        ('addresses__locality__icontains',),
    )
    list_filter = ('status', 'organization_type__category',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                ('code', 'name',),
                ('organization_type', 'organization_form',),
                ('vat_number', 'ssn',),
                ('email', 'url',),
                ('phone', 'phone2',),
                'fax',
            )
        }),
        (_('other'), {
            'classes': ('collapse',),
            'fields': ('note',),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = (
        'id', 'status', 'category', 'code', 'name', 'phone',
        'phone2', 'fax', 'get_icon_link_url', 'get_icon_link_email',
        'ordering',
    )
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = (
        'organization_type', 'organization_form',
        # 'office', 'vat', 'delivery', 'currency', 'payment_form',
    )
    inlines = (
        ContactInline, OrganizationAddressInline,
        OrganizationPhoneInline, OrganizationEmailInline, OrganizationSocialPageInline
    )

    def get_link_url(self, obj):
        """
        Return link to organization URL
        """
        if obj.url:
            return format_html(f'<a href={obj.url} target="_blank">{obj.url}</a>')
        return ""
    get_link_url.short_description = _("url")

    def get_icon_link_url(self, obj):
        """
        Return link icon to organization URL
        """
        if obj.url:
            return format_html("""<a href={href} target="_blank">x</a>""".format(href=obj.url))
        return ""
    get_icon_link_url.short_description = _("www")

    def get_link_email(self, obj):
        """
        Return link to organization email
        """
        if obj.email:
            return format_html("<a href=mailto:{email}>{email}</a>".format(email=obj.email))
        return ""
    get_link_email.short_description = _("email")

    def get_icon_link_email(self, obj):
        """
        Return link icon to organization email
        """
        if obj.email:
            return format_html("<a href=mailto:{email}>x</a>".format(email=obj.email))
        return ""
    get_icon_link_email.short_description = _("email")

    def get_organization_type(self, obj):
        """
        Return organization type name
        """
        return format_html("<b>{organization_type}</b>".format(organization_type=obj.organization_type.name))
    get_organization_type.short_description = _("type")
    get_organization_type.order_admin_field = 'organization_type__category__name'


class OrganizationEmailAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('organization__name',)
    advanced_search_fields = (
        ('status', 'organization__name__icontains',),
        ('typology', 'email__icontains',),
    )
    list_filter = ('status', 'typology', 'is_primary',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'organization', 'email', 'is_primary',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'typology', 'organization', 'email', 'is_primary', 'ordering')
    list_display_links = ('email',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('organization', 'typology',)


class OrganizationPhoneAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('organization__name',)
    advanced_search_fields = (
        ('status', 'organization__name__icontains',),
        ('typology', 'phone__icontains',),
    )
    list_filter = ('status', 'typology', 'is_primary',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'organization', 'phone', 'is_primary',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'typology', 'organization', 'phone', 'is_primary', 'ordering')
    list_display_links = ('phone',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('organization', 'typology',)


class OrganizationSocialPageAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('organization__name',)
    advanced_search_fields = (
        ('status', 'organization__name__icontains',),
        ('social', 'link__icontains',),
    )
    list_filter = ('status', 'social',)
    fieldsets = (
        (_('general information'), {
            'fields': ('social', 'organization', 'link',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'social', 'organization', 'link', 'ordering')
    list_display_links = ('social',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('organization',)


class OwnerOrganizationAdmin(OrganizationAdmin):
    pass


class ContactRoleAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name',),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'code', 'name', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class ContactTitleAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('code', 'name',)
    advanced_search_fields = (
        ('status', 'name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('code', 'name', 'abbreviation',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('code', 'status', 'name', 'abbreviation', 'ordering')
    list_display_links = ('name',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class ContactAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('first_name', 'last_name',)
    advanced_search_fields = (
        ('status', 'mistakes',),
        ('organization__organization_type__category', 'organization__name__icontains',),
        ('roles',),
        ('email__icontains',),
        ('last_name__icontains',),
        ('first_name__icontains',),
        ('date_create__gte', 'date_create__lte',),
        ('mail_profile_not_completed_sent',),
        ('agree_privacy', 'agree_terms', 'agree_advertising',),
    )
    list_filter = (
        'status', 'organization__organization_type__category',
        'mail_profile_not_completed_sent',
    )
    fieldsets = (
        (_('general information'), {
            'fields': (
                'code', 'user',
                ('last_name', 'first_name',),
                ('email', 'username',),
                'mistakes', 'language', 'organization', 'title', 'roles',
                ('agree_privacy', 'agree_terms', 'agree_advertising'),
                'mail_profile_not_completed_sent',
                'avatar',
            )
        }),
        (_('profiles'), {
            'fields': (
                ('is_superuser', 'is_staff',),
            ),
        }),
        (_('note'), {
            'fields': ('note',),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status',)
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = (
        'id', 'status', 'code', 'organization', 'last_name', 'first_name', 'email', 'mistakes',
        'agree_privacy', 'agree_terms', 'agree_advertising', 'mail_profile_not_completed_sent',
        'is_superuser', 'is_staff',
    )
    list_display_links = ('last_name', 'first_name')
    readonly_fields = (
        'creator', 'date_create', 'last_modifier', 'date_last_modify',
        'username',
    )
    autocomplete_fields = ('user', 'organization', 'roles', 'title', 'language',)


class ContactEmailAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('contact__last_name', 'contact__first_name',)
    advanced_search_fields = (
        ('status', 'contact__last_name__icontains', 'contact__first_name__icontains'),
        ('typology', 'email__icontains',),
    )
    list_filter = ('status', 'typology', 'is_primary',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'contact', 'email', 'is_primary',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'typology', 'contact', 'email', 'is_primary', 'ordering')
    list_display_links = ('email',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('contact', 'typology',)


class ContactPhoneAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('contact__last_name', 'contact__first_name',)
    advanced_search_fields = (
        ('status', 'contact__last_name__icontains', 'contact__first_name__icontains'),
        ('typology', 'phone__icontains',),
    )
    list_filter = ('status', 'typology', 'is_primary',)
    fieldsets = (
        (_('general information'), {
            'fields': ('typology', 'contact', 'phone', 'is_primary',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'typology', 'contact', 'phone', 'is_primary', 'ordering')
    list_display_links = ('phone',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('contact', 'typology',)


class ContactSocialAccountAdmin(
    StatusActionMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('contact__last_name', 'contact__first_name',)
    advanced_search_fields = (
        ('status', 'contact__last_name__icontains', 'contact__first_name__icontains'),
        ('social', 'account__icontains',),
    )
    list_filter = ('status', 'social',  'is_primary',)
    fieldsets = (
        (_('general information'), {
            'fields': ('social', 'contact', 'account', 'is_primary',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'social', 'contact', 'account', 'is_primary', 'ordering')
    list_display_links = ('social',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('contact',)


class OwnerContactAdmin(ContactAdmin):
    pass


admin.site.register(models.OrganizationAddress, OrganizationAddressAdmin)
admin.site.register(models.EmailTypology, EmailTypologyAdmin)
admin.site.register(models.PhoneTypology, PhoneTypologyAdmin)
admin.site.register(models.OrganizationType, OrganizationTypeAdmin)
admin.site.register(models.OrganizationForm, OrganizationFormAdmin)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(models.OrganizationEmail, OrganizationEmailAdmin)
admin.site.register(models.OrganizationPhone, OrganizationPhoneAdmin)
admin.site.register(models.OrganizationSocialPage, OrganizationSocialPageAdmin)
admin.site.register(models.OwnerOrganization, OwnerOrganizationAdmin)
admin.site.register(models.ContactRole, ContactRoleAdmin)
admin.site.register(models.ContactTitle, ContactTitleAdmin)
admin.site.register(Contact, ContactAdmin)
admin.site.register(models.ContactEmail, ContactEmailAdmin)
admin.site.register(models.ContactPhone, ContactPhoneAdmin)
admin.site.register(models.ContactSocialAccount, ContactSocialAccountAdmin)
admin.site.register(models.OwnerContact, OwnerContactAdmin)
