# -*- coding: utf-8 -*-
import swapper
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin
from thx.role.core.views import ThuxRoleViewSetMixin

from ... import models
from . import serializers, filters, permissions

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)

Contact = swapper.load_model('organization', 'Contact')
Organization = swapper.load_model('organization', 'Organization')


# ORGANIZATION ADDRESS
class OrganizationAddressViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationAddressViewSet
    -----------

    :action list: Return the organization address list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization address.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization address list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization address instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization address instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization address instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization address instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationAddress.objects.all()
    serializer_class = serializers.OrganizationAddressSerializer
    filter_backends = (filters.OrganizationAddressFilterBackend,)
    permission_classes = (permissions.OrganizationAddressPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="address-type-choices")
    def address_type_choices(self, request, *args, **kwargs):
        return Response(settings.ADDRESS_ADDRESS_TYPE_CHOICES)


# EMAIL TYPOLOGY
class EmailTypologyViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    EmailTypologyViewSet
    -----------

    :action list: Return the email typology list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given email typology.
        `Profiles`: `superuser`, `staff`

    :action light: Return the email typology list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new email typology instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an email typology instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an email typology instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an email typology instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.EmailTypology.objects.all()
    serializer_class = serializers.EmailTypologySerializer
    filter_backends = (filters.EmailTypologyFilterBackend,)
    permission_classes = (permissions.EmailTypologyPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an email typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an email typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PHONE TYPOLOGY
class PhoneTypologyViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    PhoneTypologyViewSet
    -----------

    :action list: Return the phone typology list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given phone typology.
        `Profiles`: `superuser`, `staff`

    :action light: Return the phone typology list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a phone typology instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a phone typology instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.PhoneTypology.objects.all()
    serializer_class = serializers.PhoneTypologySerializer
    filter_backends = (filters.PhoneTypologyFilterBackend,)
    permission_classes = (permissions.PhoneTypologyPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a phone typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a phone typology instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION TYPE
class OrganizationTypeViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationTypeViewSet
    -----------

    :action list: Return the organization type list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization type.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization type list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization type instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization type instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization type instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization type instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationType.objects.all()
    serializer_class = serializers.OrganizationTypeSerializer
    filter_backends = (filters.OrganizationTypeFilterBackend, )
    permission_classes = (permissions.OrganizationTypePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization type instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="category-choices")
    def category_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONTYPE_CATEGORY_CHOICES)


# ORGANIZATION FORM
class OrganizationFormViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationFormViewSet
    -----------

    :action list: Return the organization form list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization form.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization form list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization form instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization form instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization form instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization form instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationForm.objects.all()
    serializer_class = serializers.OrganizationFormSerializer
    filter_backends = (filters.OrganizationFormFilterBackend,)
    permission_classes = (permissions.OrganizationFormPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization form instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION
class OrganizationViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationViewSet
    -----------

    :action list: Return the organization list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = Organization.objects.all()
    serializer_class = serializers.OrganizationSerializer
    filter_backends = (filters.OrganizationFilterBackend,)
    permission_classes = (permissions.OrganizationPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# OWNER ORGANIZATION
class OwnerOrganizationViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OwnerOrganizationViewSet
    -----------

    :action list: Return the owner organization list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given owner organization.
        `Profiles`: `superuser`, `staff`

    :action light: Return the owner organization list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an owner organization instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an owner organization instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OwnerOrganization.objects.all()
    serializer_class = serializers.OwnerOrganizationSerializer
    filter_backends = (filters.OwnerOrganizationFilterBackend,)
    permission_classes = (permissions.OwnerOrganizationPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an owner organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an owner organization instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION EMAIL
class OrganizationEmailViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationEmailViewSet
    -----------

    :action list: Return the organization email list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization email.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization email list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization email instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization email instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization email instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization email instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationEmail.objects.all()
    serializer_class = serializers.OrganizationEmailSerializer
    filter_backends = (filters.OrganizationEmailFilterBackend,)
    permission_classes = (permissions.OrganizationEmailPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION PHONE
class OrganizationPhoneViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationPhoneViewSet
    -----------

    :action list: Return the organization phone list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization phone.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization phone list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization phone instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization phone instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationPhone.objects.all()
    serializer_class = serializers.OrganizationPhoneSerializer
    filter_backends = (filters.OrganizationPhoneFilterBackend,)
    permission_classes = (permissions.OrganizationPhonePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ORGANIZATION SOCIAL PAGE
class OrganizationSocialPageViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OrganizationSocialPageViewSet
    -----------

    :action list: Return the organization social page list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given organization social page.
        `Profiles`: `superuser`, `staff`

    :action light: Return the organization social page list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an organization social page instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an organization social page instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OrganizationSocialPage.objects.all()
    serializer_class = serializers.OrganizationSocialPageSerializer
    filter_backends = (filters.OrganizationSocialPageFilterBackend,)
    permission_classes = (permissions.OrganizationSocialPagePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an organization social page instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an organization social page instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONSOCIALPAGE_SOCIAL_DEF)


# CONTACT ROLE
class ContactRoleViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactRoleViewSet
    -----------

    :action list: Return the contact role list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact role.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact role list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact role instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact role instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact role instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact role instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactRole.objects.all()
    serializer_class = serializers.ContactRoleSerializer
    filter_backends = (filters.ContactRoleFilterBackend,)
    permission_classes = (permissions.ContactRolePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact role instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT TITLE
class ContactTitleViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactTitleViewSet
    -----------

    :action list: Return the contact title list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact title.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact title list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact title instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact title instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact title instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact title instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactTitle.objects.all()
    serializer_class = serializers.ContactTitleSerializer
    filter_backends = (filters.ContactTitleFilterBackend,)
    permission_classes = (permissions.ContactTitlePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact title instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact title instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT
class ContactViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactViewSet
    -----------

    :action list: Return the contact list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = Contact.objects.all()
    serializer_class = serializers.ContactSerializer
    filter_backends = (filters.ContactFilterBackend,)
    permission_classes = (permissions.ContactPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# OWNER CONTACT
class OwnerContactViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    OwnerContactViewSet
    -----------

    :action list: Return the owner contact list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given owner contact.
        `Profiles`: `superuser`, `staff`

    :action light: Return the owner contact list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update an owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch an owner contact instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy an owner contact instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.OwnerContact.objects.all()
    serializer_class = serializers.OwnerContactSerializer
    filter_backends = (filters.OwnerContactFilterBackend,)
    permission_classes = (permissions.OwnerContactPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable an owner contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable an owner contact instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT EMAIL
class ContactEmailViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactEmailViewSet
    -----------

    :action list: Return the contact email list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact email.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact email list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact email instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact email instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact email instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact email instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactEmail.objects.all()
    serializer_class = serializers.ContactEmailSerializer
    filter_backends = (filters.ContactEmailFilterBackend,)
    permission_classes = (permissions.ContactEmailPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact email instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT PHONE
class ContactPhoneViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactPhoneViewSet
    -----------

    :action list: Return the contact phone list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact phone.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact phone list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact phone instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact phone instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactPhone.objects.all()
    serializer_class = serializers.ContactPhoneSerializer
    filter_backends = (filters.ContactPhoneFilterBackend,)
    permission_classes = (permissions.ContactPhonePermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact phone instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# CONTACT SOCIAL ACCOUNT
class ContactSocialAccountViewSet(ThuxRoleViewSetMixin, ThuxModelMixin, viewsets.ModelViewSet):
    """
    ContactSocialAccountViewSet
    -----------

    :action list: Return the contact social account list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given contact social account.
        `Profiles`: `superuser`, `staff`

    :action light: Return the contact social account list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a contact social account instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Destroy a contact social account instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ContactSocialAccount.objects.all()
    serializer_class = serializers.ContactSocialAccountSerializer
    filter_backends = (filters.ContactSocialAccountFilterBackend,)
    permission_classes = (permissions.ContactSocialAccountPermission,)

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a contact social account instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a contact social account instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_CONTACTSOCIALACCOUNT_SOCIAL_CHOICES)
