# -*- coding: utf-8 -*-
import swapper
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel

from .. import managers
from .abstracts import AbstractContact
from .organizations import EmailTypology, PhoneTypology


class ContactRole(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        unique=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _('contact role')
        verbose_name_plural = _('contact roles')
        ordering = ("ordering", "name")
        permissions = (
            ("list_contactrole", "Can list contact role"),
            ("detail_contactrole", "Can detail contact role"),
            ("enable_contactrole", "Can enable contact role"),
            ("disable_contactrole", "Can disable contact role"),
        )

    def __str__(self):
        return self.name


class ContactTitle(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        primary_key=True,
        verbose_name=_("code"),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("name"),
    )
    abbreviation = models.CharField(
        max_length=10,
        blank=True,
        verbose_name=_("abbreviation"),
    )

    class Meta:
        verbose_name = _('contact title')
        verbose_name_plural = _('contact titles')
        unique_together = (('name', 'abbreviation'),)
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_contacttitle", "Can list contact title"),
            ("detail_contacttitle", "Can detail contact title"),
            ("enable_contacttitle", "Can enable contact title"),
            ("disable_contacttitle", "Can disable contact title"),
        )

    def __str__(self):
        return self.name


class Contact(AbstractContact):
    class Meta:
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')
        ordering = ('ordering', 'user__last_name', 'user__first_name')
        permissions = (
            ("list_contact", "Can list contact"),
            ("detail_contact", "Can detail contact"),
            ("enable_contact", "Can enable contact"),
            ("disable_contact", "Can disable contact"),
        )
        swappable = swapper.swappable_setting('organization', 'Contact')


if not swapper.is_swapped("organization", "Contact"):
    class OwnerContact(Contact):
        objects = managers.OwnerContactManager()

        class Meta:
            proxy = True
            verbose_name = _('owner contact')
            verbose_name_plural = _('owner contacts')
            permissions = (
                ("list_ownercontact", "Can list owner contact"),
                ("detail_ownercontact", "Can detail owner contact"),
                ("enable_ownercontact", "Can enable owner contact"),
                ("disable_ownercontact", "Can disable owner contact"),
            )


class ContactEmail(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    contact = models.ForeignKey(
        swapper.get_model_name('organization', 'Contact'),
        on_delete=models.PROTECT,
        related_name='emails',
        verbose_name=_('contact'),
    )
    typology = models.ForeignKey(
        EmailTypology,
        on_delete=models.PROTECT,
        related_name='contact_emails',
        verbose_name=_('typology'),
    )
    email = models.EmailField(
        max_length=255,
        verbose_name=_("email"),
    )
    is_primary = models.BooleanField(
        default=False,
        verbose_name=_('is primary')
    )

    class Meta:
        verbose_name = _('contact email')
        verbose_name_plural = _('contact emails')
        unique_together = (('contact', 'email',),)
        ordering = ('ordering', 'email',)
        permissions = (
            ("list_contactemail", "Can list contact email"),
            ("detail_contactemail", "Can detail contact email"),
            ("enable_contactemail", "Can enable contact email"),
            ("disable_contactemail", "Can disable contact email"),
        )

    def __str__(self):
        return '{}: {}'.format(self.typology, self.email)

    def save(self, *args, **kwargs):
        if self.is_primary:
            prim_emails = ContactEmail.objects.filter(contact=self.contact, is_primary=True)
            for email in prim_emails:
                email.is_primary = False
                email.save()
        super().save(*args, **kwargs)

    @property
    def contact_full_name(self) -> str:
        return self.contact.full_name

    @property
    def contact_last_name(self) -> str:
        return self.contact.last_name

    @property
    def contact_first_name(self) -> str:
        return self.contact.first_name

    @property
    def typology_name(self) -> str:
        return self.typology.name


class ContactPhone(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    contact = models.ForeignKey(
        swapper.get_model_name('organization', 'Contact'),
        on_delete=models.PROTECT,
        related_name='phones',
        verbose_name=_('contact'),
    )
    typology = models.ForeignKey(
        PhoneTypology,
        on_delete=models.PROTECT,
        related_name='contact_phones',
        verbose_name=_('typology'),
    )
    phone = models.CharField(
        max_length=20,
        verbose_name=_("phone"),
    )
    is_primary = models.BooleanField(
        default=False,
        verbose_name=_('is primary')
    )

    class Meta:
        verbose_name = _('contact phone')
        verbose_name_plural = _('contact phones')
        unique_together = (('contact', 'phone',),)
        ordering = ('ordering', 'phone',)
        permissions = (
            ("list_contactphone", "Can list contact phone"),
            ("detail_contactphone", "Can detail contact phone"),
            ("enable_contactphone", "Can enable contact phone"),
            ("disable_contactphone", "Can disable contact phone"),
        )

    def __str__(self):
        return '{}: {}'.format(self.typology, self.phone)

    def save(self, *args, **kwargs):
        if self.is_primary:
            prim_phones = ContactPhone.objects.filter(
                contact=self.contact, is_primary=True
            )
            for phone in prim_phones:
                phone.is_primary = False
                phone.save()
        super().save(*args, **kwargs)

    @property
    def contact_full_name(self) -> str:
        return self.contact.full_name

    @property
    def contact_last_name(self) -> str:
        return self.contact.last_name

    @property
    def contact_first_name(self) -> str:
        return self.contact.first_name

    @property
    def typology_name(self) -> str:
        return self.typology.name


class ContactSocialAccount(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    contact = models.ForeignKey(
        swapper.get_model_name('organization', 'Contact'),
        on_delete=models.PROTECT,
        related_name='accounts',
        verbose_name=_('contact'),
    )
    social = models.CharField(
        max_length=255,
        choices=settings.ORGANIZATION_CONTACTSOCIALACCOUNT_SOCIAL_CHOICES,
        db_index=True,
        verbose_name=_('social'),
    )
    account = models.CharField(
        max_length=255,
        verbose_name=_("account"),
    )
    is_primary = models.BooleanField(
        default=False,
        verbose_name=_('is primary')
    )

    class Meta:
        verbose_name = _('contact social account')
        verbose_name_plural = _('contact social accounts')
        unique_together = (('social', 'account',),)
        ordering = ('ordering', 'account',)
        permissions = (
            ("list_contactsocialaccount", "Can list contact social account"),
            ("detail_contactsocialaccount", "Can detail contact social account"),
            ("enable_contactsocialaccount", "Can enable contact social account"),
            ("disable_contactsocialaccount", "Can disable contact social account"),
        )

    def __str__(self):
        return '{}: {}'.format(self.get_social_display(), self.account)

    def save(self, *args, **kwargs):
        if self.is_primary:
            prim_accounts = ContactSocialAccount.objects.filter(
                contact=self.contact, is_primary=True
            )
            for account in prim_accounts:
                account.is_primary = False
                account.save()
        super().save(*args, **kwargs)

    @property
    def contact_full_name(self) -> str:
        return self.contact.full_name

    @property
    def contact_last_name(self) -> str:
        return self.contact.last_name

    @property
    def contact_first_name(self) -> str:
        return self.contact.first_name
